# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'designer/window.ui'
#
# Created by: PyQt5 UI code generator 5.15.6
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Window(object):
    def setupUi(self, Window):
        Window.setObjectName("Window")
        Window.resize(1112, 874)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/icons/app.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        Window.setWindowIcon(icon)
        self._central_widget = QtWidgets.QWidget(Window)
        self._central_widget.setObjectName("_central_widget")
        self._central_layout = QtWidgets.QGridLayout(self._central_widget)
        self._central_layout.setObjectName("_central_layout")
        self.centralSplitter = QtWidgets.QSplitter(self._central_widget)
        self.centralSplitter.setOrientation(QtCore.Qt.Horizontal)
        self.centralSplitter.setChildrenCollapsible(False)
        self.centralSplitter.setObjectName("centralSplitter")
        self.leftSplitter = QtWidgets.QSplitter(self.centralSplitter)
        self.leftSplitter.setOrientation(QtCore.Qt.Vertical)
        self.leftSplitter.setChildrenCollapsible(False)
        self.leftSplitter.setObjectName("leftSplitter")
        self.filters = QtWidgets.QTreeWidget(self.leftSplitter)
        self.filters.setAllColumnsShowFocus(True)
        self.filters.setExpandsOnDoubleClick(False)
        self.filters.setObjectName("filters")
        self.layoutWidget = QtWidgets.QWidget(self.leftSplitter)
        self.layoutWidget.setObjectName("layoutWidget")
        self._search_layout = QtWidgets.QGridLayout(self.layoutWidget)
        self._search_layout.setContentsMargins(0, 0, 0, 0)
        self._search_layout.setObjectName("_search_layout")
        self.searchBox = QtWidgets.QComboBox(self.layoutWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.searchBox.sizePolicy().hasHeightForWidth())
        self.searchBox.setSizePolicy(sizePolicy)
        self.searchBox.setEditable(True)
        self.searchBox.setMaxCount(30)
        self.searchBox.setInsertPolicy(QtWidgets.QComboBox.InsertAtTop)
        self.searchBox.setObjectName("searchBox")
        self._search_layout.addWidget(self.searchBox, 0, 0, 1, 1)
        self._button_layout = QtWidgets.QHBoxLayout()
        self._button_layout.setObjectName("_button_layout")
        self.searchFiles = QtWidgets.QToolButton(self.layoutWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.searchFiles.sizePolicy().hasHeightForWidth())
        self.searchFiles.setSizePolicy(sizePolicy)
        self.searchFiles.setObjectName("searchFiles")
        self._button_layout.addWidget(self.searchFiles)
        self.searchButton = QtWidgets.QToolButton(self.layoutWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.searchButton.sizePolicy().hasHeightForWidth())
        self.searchButton.setSizePolicy(sizePolicy)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/icons/search.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.searchButton.setIcon(icon1)
        self.searchButton.setObjectName("searchButton")
        self._button_layout.addWidget(self.searchButton)
        self._search_layout.addLayout(self._button_layout, 0, 1, 1, 1)
        self.searchOptions = QtWidgets.QWidget(self.layoutWidget)
        self.searchOptions.setObjectName("searchOptions")
        self._options_layout = QtWidgets.QVBoxLayout(self.searchOptions)
        self._options_layout.setObjectName("_options_layout")
        self.keyGroup = QtWidgets.QGroupBox(self.searchOptions)
        self.keyGroup.setObjectName("keyGroup")
        self.gridLayout = QtWidgets.QGridLayout(self.keyGroup)
        self.gridLayout.setObjectName("gridLayout")
        self.keyNames = QtWidgets.QCheckBox(self.keyGroup)
        self.keyNames.setObjectName("keyNames")
        self.gridLayout.addWidget(self.keyNames, 0, 0, 1, 1)
        self.keyProvides = QtWidgets.QCheckBox(self.keyGroup)
        self.keyProvides.setObjectName("keyProvides")
        self.gridLayout.addWidget(self.keyProvides, 0, 1, 1, 1)
        self.keyDescriptions = QtWidgets.QCheckBox(self.keyGroup)
        self.keyDescriptions.setObjectName("keyDescriptions")
        self.gridLayout.addWidget(self.keyDescriptions, 1, 0, 1, 1)
        self.keyReplaces = QtWidgets.QCheckBox(self.keyGroup)
        self.keyReplaces.setObjectName("keyReplaces")
        self.gridLayout.addWidget(self.keyReplaces, 1, 1, 1, 1)
        self.keyDepends = QtWidgets.QCheckBox(self.keyGroup)
        self.keyDepends.setObjectName("keyDepends")
        self.gridLayout.addWidget(self.keyDepends, 2, 0, 1, 1)
        self.keyOptional = QtWidgets.QCheckBox(self.keyGroup)
        self.keyOptional.setEnabled(True)
        self.keyOptional.setObjectName("keyOptional")
        self.gridLayout.addWidget(self.keyOptional, 2, 1, 1, 1)
        self.keyMaintainer = QtWidgets.QCheckBox(self.keyGroup)
        self.keyMaintainer.setObjectName("keyMaintainer")
        self.gridLayout.addWidget(self.keyMaintainer, 3, 0, 1, 1)
        self.keyFiles = QtWidgets.QCheckBox(self.keyGroup)
        self.keyFiles.setEnabled(True)
        self.keyFiles.setObjectName("keyFiles")
        self.gridLayout.addWidget(self.keyFiles, 3, 1, 1, 1)
        self._options_layout.addWidget(self.keyGroup)
        self.scopeGroup = QtWidgets.QGroupBox(self.searchOptions)
        self.scopeGroup.setObjectName("scopeGroup")
        self._scope_layout = QtWidgets.QGridLayout(self.scopeGroup)
        self._scope_layout.setObjectName("_scope_layout")
        spacerItem = QtWidgets.QSpacerItem(0, 0, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self._scope_layout.addItem(spacerItem, 3, 0, 1, 2)
        self.scopeAll = QtWidgets.QRadioButton(self.scopeGroup)
        self.scopeAll.setObjectName("scopeAll")
        self._scope_layout.addWidget(self.scopeAll, 0, 0, 1, 1)
        self.scopeInstalled = QtWidgets.QRadioButton(self.scopeGroup)
        self.scopeInstalled.setObjectName("scopeInstalled")
        self._scope_layout.addWidget(self.scopeInstalled, 1, 0, 1, 1)
        self.scopeNonInstalled = QtWidgets.QRadioButton(self.scopeGroup)
        self.scopeNonInstalled.setObjectName("scopeNonInstalled")
        self._scope_layout.addWidget(self.scopeNonInstalled, 2, 0, 1, 1)
        self.scopePackages = QtWidgets.QRadioButton(self.scopeGroup)
        self.scopePackages.setObjectName("scopePackages")
        self._scope_layout.addWidget(self.scopePackages, 0, 1, 1, 1)
        self.scopeGroups = QtWidgets.QRadioButton(self.scopeGroup)
        self.scopeGroups.setObjectName("scopeGroups")
        self._scope_layout.addWidget(self.scopeGroups, 1, 1, 1, 1)
        self.scopeAUR = QtWidgets.QCheckBox(self.scopeGroup)
        self.scopeAUR.setObjectName("scopeAUR")
        self._scope_layout.addWidget(self.scopeAUR, 2, 1, 1, 1)
        self._options_layout.addWidget(self.scopeGroup)
        self._search_layout.addWidget(self.searchOptions, 1, 0, 1, 2)
        self._search_layout.setColumnStretch(0, 1)
        self._search_layout.setRowStretch(1, 1)
        self.rightSplitter = QtWidgets.QSplitter(self.centralSplitter)
        self.rightSplitter.setOrientation(QtCore.Qt.Vertical)
        self.rightSplitter.setChildrenCollapsible(False)
        self.rightSplitter.setObjectName("rightSplitter")
        self.packages = QtWidgets.QTreeView(self.rightSplitter)
        self.packages.setStyleSheet("QTreeView::item {\n"
"    padding-right: 15px;\n"
"}")
        self.packages.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.packages.setUniformRowHeights(True)
        self.packages.setAllColumnsShowFocus(True)
        self.packages.setObjectName("packages")
        self.packages.header().setSortIndicatorShown(True)
        self.information = QtWidgets.QTabWidget(self.rightSplitter)
        self.information.setObjectName("information")
        self.detailsTab = QtWidgets.QWidget()
        self.detailsTab.setObjectName("detailsTab")
        self._details_layout = QtWidgets.QVBoxLayout(self.detailsTab)
        self._details_layout.setContentsMargins(1, 1, 1, 1)
        self._details_layout.setSpacing(0)
        self._details_layout.setObjectName("_details_layout")
        self.details = QtWidgets.QTextBrowser(self.detailsTab)
        self.details.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.details.setOpenLinks(False)
        self.details.setObjectName("details")
        self._details_layout.addWidget(self.details)
        self.findBar = QtWidgets.QWidget(self.detailsTab)
        self.findBar.setObjectName("findBar")
        self._findbar_layout = QtWidgets.QHBoxLayout(self.findBar)
        self._findbar_layout.setContentsMargins(-1, 1, -1, 1)
        self._findbar_layout.setObjectName("_findbar_layout")
        self.findLabel = QtWidgets.QLabel(self.findBar)
        self.findLabel.setObjectName("findLabel")
        self._findbar_layout.addWidget(self.findLabel)
        self.findEdit = QtWidgets.QLineEdit(self.findBar)
        self.findEdit.setMinimumSize(QtCore.QSize(100, 0))
        self.findEdit.setObjectName("findEdit")
        self._findbar_layout.addWidget(self.findEdit)
        self.findNext = QtWidgets.QToolButton(self.findBar)
        self.findNext.setEnabled(False)
        self.findNext.setAutoRaise(True)
        self.findNext.setObjectName("findNext")
        self._findbar_layout.addWidget(self.findNext)
        self.findPrevious = QtWidgets.QToolButton(self.findBar)
        self.findPrevious.setEnabled(False)
        self.findPrevious.setAutoRaise(True)
        self.findPrevious.setObjectName("findPrevious")
        self._findbar_layout.addWidget(self.findPrevious)
        self.findCase = QtWidgets.QCheckBox(self.findBar)
        self.findCase.setObjectName("findCase")
        self._findbar_layout.addWidget(self.findCase)
        self.findClose = QtWidgets.QToolButton(self.findBar)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/icons/close.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.findClose.setIcon(icon2)
        self.findClose.setAutoRaise(True)
        self.findClose.setObjectName("findClose")
        self._findbar_layout.addWidget(self.findClose)
        self._findbar_layout.setStretch(1, 1)
        self._findbar_layout.setStretch(4, 1)
        self._details_layout.addWidget(self.findBar)
        self.information.addTab(self.detailsTab, "")
        self.filesTab = QtWidgets.QWidget()
        self.filesTab.setObjectName("filesTab")
        self._files_layout = QtWidgets.QVBoxLayout(self.filesTab)
        self._files_layout.setContentsMargins(1, 1, 1, 1)
        self._files_layout.setSpacing(0)
        self._files_layout.setObjectName("_files_layout")
        self.files = QtWidgets.QTextBrowser(self.filesTab)
        self.files.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.files.setOpenLinks(False)
        self.files.setObjectName("files")
        self._files_layout.addWidget(self.files)
        self.information.addTab(self.filesTab, "")
        self.treeTab = QtWidgets.QWidget()
        self.treeTab.setObjectName("treeTab")
        self._tree_layout = QtWidgets.QVBoxLayout(self.treeTab)
        self._tree_layout.setContentsMargins(1, 1, 1, 1)
        self._tree_layout.setSpacing(0)
        self._tree_layout.setObjectName("_tree_layout")
        self.tree = QtWidgets.QTextBrowser(self.treeTab)
        self.tree.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.tree.setOpenLinks(False)
        self.tree.setObjectName("tree")
        self._tree_layout.addWidget(self.tree)
        self.information.addTab(self.treeTab, "")
        self.logTab = QtWidgets.QWidget()
        self.logTab.setObjectName("logTab")
        self._log_layout = QtWidgets.QVBoxLayout(self.logTab)
        self._log_layout.setContentsMargins(1, 1, 1, 1)
        self._log_layout.setSpacing(0)
        self._log_layout.setObjectName("_log_layout")
        self.log = QtWidgets.QTextBrowser(self.logTab)
        self.log.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.log.setOpenLinks(False)
        self.log.setObjectName("log")
        self._log_layout.addWidget(self.log)
        self.information.addTab(self.logTab, "")
        self.cacheTab = QtWidgets.QWidget()
        self.cacheTab.setObjectName("cacheTab")
        self._cache_layout = QtWidgets.QVBoxLayout(self.cacheTab)
        self._cache_layout.setContentsMargins(1, 1, 1, 1)
        self._cache_layout.setSpacing(0)
        self._cache_layout.setObjectName("_cache_layout")
        self.cache = QtWidgets.QTextBrowser(self.cacheTab)
        self.cache.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.cache.setOpenLinks(False)
        self.cache.setObjectName("cache")
        self._cache_layout.addWidget(self.cache)
        self.information.addTab(self.cacheTab, "")
        self.backupTab = QtWidgets.QWidget()
        self.backupTab.setObjectName("backupTab")
        self._backup_layout = QtWidgets.QVBoxLayout(self.backupTab)
        self._backup_layout.setContentsMargins(1, 1, 1, 1)
        self._backup_layout.setSpacing(0)
        self._backup_layout.setObjectName("_backup_layout")
        self.backup = QtWidgets.QTextBrowser(self.backupTab)
        self.backup.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.backup.setOpenLinks(False)
        self.backup.setObjectName("backup")
        self._backup_layout.addWidget(self.backup)
        self.information.addTab(self.backupTab, "")
        self._central_layout.addWidget(self.centralSplitter, 0, 0, 1, 1)
        Window.setCentralWidget(self._central_widget)
        self._menubar = QtWidgets.QMenuBar(Window)
        self._menubar.setGeometry(QtCore.QRect(0, 0, 1112, 22))
        self._menubar.setObjectName("_menubar")
        self.fileMenu = QtWidgets.QMenu(self._menubar)
        self.fileMenu.setObjectName("fileMenu")
        self.helpMenu = QtWidgets.QMenu(self._menubar)
        self.helpMenu.setObjectName("helpMenu")
        self.toolsMenu = QtWidgets.QMenu(self._menubar)
        self.toolsMenu.setObjectName("toolsMenu")
        Window.setMenuBar(self._menubar)
        self._statusbar = QtWidgets.QStatusBar(Window)
        self._statusbar.setObjectName("_statusbar")
        Window.setStatusBar(self._statusbar)
        self.fileRefresh = QtWidgets.QAction(Window)
        self.fileRefresh.setObjectName("fileRefresh")
        self.fileQuit = QtWidgets.QAction(Window)
        self.fileQuit.setObjectName("fileQuit")
        self.helpAbout = QtWidgets.QAction(Window)
        self.helpAbout.setObjectName("helpAbout")
        self.helpAboutQt = QtWidgets.QAction(Window)
        self.helpAboutQt.setObjectName("helpAboutQt")
        self.helpManual = QtWidgets.QAction(Window)
        self.helpManual.setObjectName("helpManual")
        self.fileOffline = QtWidgets.QAction(Window)
        self.fileOffline.setCheckable(True)
        self.fileOffline.setObjectName("fileOffline")
        self.toolsCopy = QtWidgets.QAction(Window)
        self.toolsCopy.setObjectName("toolsCopy")
        self.toolsStatistics = QtWidgets.QAction(Window)
        self.toolsStatistics.setObjectName("toolsStatistics")
        self.fileCancel = QtWidgets.QAction(Window)
        self.fileCancel.setObjectName("fileCancel")
        self.toolsClear = QtWidgets.QAction(Window)
        self.toolsClear.setObjectName("toolsClear")
        self.fileMenu.addAction(self.fileOffline)
        self.fileMenu.addAction(self.fileRefresh)
        self.fileMenu.addAction(self.fileCancel)
        self.fileMenu.addSeparator()
        self.fileMenu.addAction(self.fileQuit)
        self.helpMenu.addAction(self.helpManual)
        self.helpMenu.addSeparator()
        self.helpMenu.addAction(self.helpAbout)
        self.helpMenu.addAction(self.helpAboutQt)
        self.toolsMenu.addAction(self.toolsStatistics)
        self.toolsMenu.addAction(self.toolsCopy)
        self.toolsMenu.addSeparator()
        self.toolsMenu.addAction(self.toolsClear)
        self._menubar.addAction(self.fileMenu.menuAction())
        self._menubar.addAction(self.toolsMenu.menuAction())
        self._menubar.addAction(self.helpMenu.menuAction())
        self.findLabel.setBuddy(self.findEdit)

        self.retranslateUi(Window)
        QtCore.QMetaObject.connectSlotsByName(Window)
        Window.setTabOrder(self.filters, self.searchBox)
        Window.setTabOrder(self.searchBox, self.searchFiles)
        Window.setTabOrder(self.searchFiles, self.searchButton)
        Window.setTabOrder(self.searchButton, self.keyNames)
        Window.setTabOrder(self.keyNames, self.keyDescriptions)
        Window.setTabOrder(self.keyDescriptions, self.keyDepends)
        Window.setTabOrder(self.keyDepends, self.keyMaintainer)
        Window.setTabOrder(self.keyMaintainer, self.keyProvides)
        Window.setTabOrder(self.keyProvides, self.keyReplaces)
        Window.setTabOrder(self.keyReplaces, self.keyOptional)
        Window.setTabOrder(self.keyOptional, self.keyFiles)
        Window.setTabOrder(self.keyFiles, self.scopeAll)
        Window.setTabOrder(self.scopeAll, self.scopeInstalled)
        Window.setTabOrder(self.scopeInstalled, self.scopeNonInstalled)
        Window.setTabOrder(self.scopeNonInstalled, self.scopePackages)
        Window.setTabOrder(self.scopePackages, self.scopeGroups)
        Window.setTabOrder(self.scopeGroups, self.scopeAUR)
        Window.setTabOrder(self.scopeAUR, self.packages)
        Window.setTabOrder(self.packages, self.information)
        Window.setTabOrder(self.information, self.details)
        Window.setTabOrder(self.details, self.files)
        Window.setTabOrder(self.files, self.tree)
        Window.setTabOrder(self.tree, self.log)
        Window.setTabOrder(self.log, self.cache)
        Window.setTabOrder(self.cache, self.findEdit)
        Window.setTabOrder(self.findEdit, self.findNext)
        Window.setTabOrder(self.findNext, self.findPrevious)
        Window.setTabOrder(self.findPrevious, self.findCase)
        Window.setTabOrder(self.findCase, self.findClose)

    def retranslateUi(self, Window):
        _translate = QtCore.QCoreApplication.translate
        self.filters.headerItem().setText(0, _translate("Window", "Filters"))
        self.searchFiles.setText(_translate("Window", "..."))
        self.keyGroup.setTitle(_translate("Window", "&Keys"))
        self.keyNames.setText(_translate("Window", "Names"))
        self.keyProvides.setText(_translate("Window", "Provides"))
        self.keyDescriptions.setText(_translate("Window", "Descriptions"))
        self.keyReplaces.setText(_translate("Window", "Replaces"))
        self.keyDepends.setText(_translate("Window", "Dependencies"))
        self.keyOptional.setText(_translate("Window", "Optional"))
        self.keyMaintainer.setText(_translate("Window", "Maintainer"))
        self.keyFiles.setText(_translate("Window", "Files"))
        self.scopeGroup.setTitle(_translate("Window", "&Scope"))
        self.scopeAll.setText(_translate("Window", "All"))
        self.scopeInstalled.setText(_translate("Window", "Installed"))
        self.scopeNonInstalled.setText(_translate("Window", "Non-Installed"))
        self.scopePackages.setText(_translate("Window", "Packages"))
        self.scopeGroups.setText(_translate("Window", "Groups"))
        self.scopeAUR.setText(_translate("Window", "Include AUR"))
        self.findLabel.setText(_translate("Window", "F&ind:"))
        self.findNext.setText(_translate("Window", "&Next"))
        self.findPrevious.setText(_translate("Window", "&Previous"))
        self.findCase.setText(_translate("Window", "Match &Case"))
        self.information.setTabText(self.information.indexOf(self.detailsTab), _translate("Window", "Details"))
        self.information.setTabText(self.information.indexOf(self.filesTab), _translate("Window", "Files"))
        self.information.setTabText(self.information.indexOf(self.treeTab), _translate("Window", "Tree"))
        self.information.setTabText(self.information.indexOf(self.logTab), _translate("Window", "Log"))
        self.information.setTabText(self.information.indexOf(self.cacheTab), _translate("Window", "Cache"))
        self.information.setTabText(self.information.indexOf(self.backupTab), _translate("Window", "Backup"))
        self.fileMenu.setTitle(_translate("Window", "&File"))
        self.helpMenu.setTitle(_translate("Window", "&Help"))
        self.toolsMenu.setTitle(_translate("Window", "&Tools"))
        self.fileRefresh.setText(_translate("Window", "&Refresh"))
        self.fileQuit.setText(_translate("Window", "&Quit"))
        self.helpAbout.setText(_translate("Window", "&About"))
        self.helpAboutQt.setText(_translate("Window", "About &Qt"))
        self.helpManual.setText(_translate("Window", "&Manual"))
        self.fileOffline.setText(_translate("Window", "&Work Offline"))
        self.toolsCopy.setText(_translate("Window", "&Copy List"))
        self.toolsStatistics.setText(_translate("Window", "Show &Statistics"))
        self.fileCancel.setText(_translate("Window", "Cancel &Task"))
        self.toolsClear.setText(_translate("Window", "Clear &Bookmarks"))
from . import resources_rc



# Copyright (C) 2010-2022, kachelaqa <kachelaqa@gmail.com>

import sys, os, getopt
from PyQt5 import QtWidgets
from PyQt5.QtCore import (
    QSettings,
    )
from PyQt5.QtWidgets import (
    QApplication,
    )


QApplication.setApplicationName('PkgBrowser')
QApplication.setApplicationVersion('0.27')

QSettings.setPath(QSettings.NativeFormat, QSettings.SystemScope, '/etc')


class Application(QApplication):

    @staticmethod
    def applicationName():
        return QApplication.applicationName().lower()

    @staticmethod
    def applicationTitle():
        return QApplication.applicationName()

    @classmethod
    def applicationUrl(cls, *paths):
        url = 'https://osdn.net/projects/%s' % cls.applicationName()
        if paths:
            url = os.path.join(url, *paths)
        return url

    @classmethod
    def settings(cls):
        organization = application = cls.applicationName()
        return Settings(organization, application)

    @classmethod
    def cacheDirectory(cls):
        settings = cls.settings()
        path = settings.value('options/cache-directory')
        if not path:
            path = os.path.join('/var/cache', cls.applicationName())
        settings.deleteLater()
        return path

    def __init__(self):
        QApplication.__init__(self, sys.argv[:])
        self._window = None

    def window(self):
        if self._window is None:
            from pkgbrowser.window import Window
            self._window = Window()
        return self._window


class Settings(QSettings):

    def value(self, key, default=None, typeinfo=None):
        if default is not None and typeinfo is None:
            if isinstance(default, list):
                typeinfo = 'QStringList'
            else:
                typeinfo = type(default)
        try:
            if typeinfo is None:
                return QSettings.value(self, key, default)
            return QSettings.value(self, key, default, typeinfo)
        except TypeError:
            print('WARNING: config key has an invalid type: %s/%s' % (
                  self.group(), key))
            return default


def usage():
    print(QApplication.translate('usage', """
usage: %s [opts]

options:
 -h  display this help and exit
 -V  display version information
 -u  create/update files cache
""" % Application.applicationName()))

def run():
    keys = 'hVu'
    try:
        options, args = getopt.getopt(sys.argv[1:], keys)
    except getopt.GetoptError as exception:
        print(':: ERROR:', exception)
        usage()
        return 2
    else:
        options = dict(options)
        if '-h' in options:
            usage()
        elif '-V' in options:
            print('%s-%s' % (
                Application.applicationTitle(),
                Application.applicationVersion(),
                ))
        elif '-u' in options:
            from pkgbrowser.conf import update_cache
            return update_cache(Application.cacheDirectory(),
                                Application.applicationTitle())
        elif QApplication.instance() is None:
            app = QtWidgets.qApp = Application()
            app.window().setup()
            app.window().show()
            return app.exec_()
        return 0



# Copyright (C) 2010-2022, kachelaqa <kachelaqa@gmail.com>

import os, re, glob, time, signal
import urllib.request
from email.utils import parsedate
from tarfile import TarFile, TarError
from zipfile import ZipFile, BadZipfile, ZIP_DEFLATED
from tempfile import TemporaryFile
from collections import defaultdict
from io import BytesIO
from multiprocessing import Pool, TimeoutError
from pkgbrowser import utils


PM_ROOT_DIR = '/'
PM_DB_PATH = '/var/lib/pacman'
PM_CONF_FILE = '/etc/pacman.conf'
PM_LOG_FILE = '/var/log/pacman.log'
PM_CACHE_DIRS = ('/var/cache/pacman/pkg',)

match_pkgfile = re.compile(r"""
    ^(.+)-([^-\s]+-[^-\s]+)-(i686|x86_64|any)
    \.pkg\.tar(?:\.\w+)?$
    """, re.X).match


def read_config(path=PM_CONF_FILE, section=None, config=None):
    if config is None:
        config = {
            'Repositories': [],
            'Servers': defaultdict(list),
            'SigLevels': defaultdict(list),
            }
    repeating = set((
        'Architecture', 'CacheDir', 'CleanMethod', 'HoldPkg',
        'IgnoreGroup', 'IgnorePkg', 'NoExtract', 'NoUpgrade',
        'SigLevel', 'LocalFileSigLevel', 'RemoteFileSigLevel',
        ))
    with open(path, 'r') as stream:
        for line in stream:
            line = line.split('#', 1)[0].strip()
            if not line:
                continue
            if line[0] == '[' and line[-1] == ']':
                section = line[1:-1]
                continue
            if not section:
                continue
            key, sep, value = line.partition('=')
            key = key.rstrip()
            value = value.lstrip()
            if not key:
                continue
            if key == 'Include':
                for path in glob.glob(value):
                    read_config(path, section, config)
            elif section == 'options':
                if value:
                    if key in repeating:
                        if key == 'CacheDir':
                            value = [value]
                        else:
                            if key == 'Architecture':
                                value = re.sub(
                                    r'\bauto\b', value, os.uname()[4])
                            value = value.split()
                        if key in config:
                            config[key].extend(value)
                            continue
                else:
                    value = True
                config[key] = value
            elif key == 'Server':
                server = value.replace('$repo', section)
                arches = config.get('Architecture')
                if arches:
                    server = server.replace('$arch', arches[0])
                config['Servers'][section].append(server)
                if section not in config['Repositories']:
                    config['Repositories'].append(section)
            elif key == 'SigLevel':
                config['SigLevels'][section].extend(value.split())
    return config

def load_logfile(path=PM_LOG_FILE):
    log = defaultdict(list)
    pattern = re.compile(r"""
        ^(\[[^]]+\])\ +(?:\[[^]]+\]\ +)?
        (installed|upgraded|downgraded|removed)\ +(\S+)\ +(\(.+)
        """, re.X | re.S)
    with open(path, 'r') as stream:
        for line in stream:
            match = pattern.match(line)
            if match is not None:
                log[match.group(3)].append(' '.join(match.groups()))
    temp = TemporaryFile()
    with ZipFile(temp, 'w') as zip:
        for name, lines in log.items():
            zip.writestr(name, ''.join(lines).encode('utf-8'))
    return ZipFile(temp)

def load_pkgcache(caches=PM_CACHE_DIRS):
    packages = defaultdict(list)
    for cache in sorted(caches):
        try:
            filenames = os.listdir(cache)
        except OSError:
            pass
        else:
            for filename in filenames:
                match = match_pkgfile(filename)
                if match is not None:
                    path = os.path.join(cache, filename)
                    if os.path.isfile(path):
                        packages['%s/%s' % match.group(1, 3)].append(path)
    temp = TemporaryFile()
    with ZipFile(temp, 'w') as zip:
        for path, filenames in packages.items():
            zip.writestr(path, '\n'.join(filenames).encode('utf-8'))
    return ZipFile(temp)

def load_srcinfo(pkgname, data):
    if isinstance(data, bytes):
        data = data.decode('utf-8', 'replace')
    keys = {
        'name': True,
        'version': False,
        'description': False,
        'url': False,
        'license': True,
        'groups': True,
        'arch': True,
        'conflicts': True,
        'depends': True,
        'makedepends': True,
        'optdepends': True,
        'provides': True,
        'replaces': True,
        }
    pkgbase = {}
    srcinfo = {}
    info = None
    for line in data.splitlines():
        key, sep, value = [part.strip() for part in line.partition('=')]
        if line.startswith('\t'):
            if info is not None and key in keys:
                if keys[key]:
                    if key in info:
                        info[key].append(value)
                    else:
                        info[key] = [value]
                else:
                    info[key] = value
        elif key == 'pkgbase':
            pkgbase.clear()
            info = pkgbase
        elif key == 'pkgname' and value == pkgname:
            info = srcinfo
    srcinfo.update(pkgbase)
    return srcinfo

def _process_archive(args):
    path, root, urls, comment = args
    archive = None
    timestamp = 0
    try:
        zip = ZipFile(path)
        try:
            if zip.comment == comment:
                timestamp = os.path.getmtime(path)
        finally:
            zip.close()
    except (EnvironmentError, BadZipfile):
        pass
    for url in urls:
        url = utils.make_url(url)
        try:
            response = urllib.request.urlopen(url, timeout=30)
            current = parsedate(response.info().get('last-modified'))
            if current is not None:
                current = time.mktime(current)
            try:
                if current is None or current > timestamp:
                    archive = BytesIO(response.read())
                else:
                    print(':: already up to date: [%s.files]' % root)
                    return True
            finally:
                response.close()
        except IOError:
            pass
        else:
            print(':: download succeeded: [%s.files] (%s)' % (root, url))
            break
    if archive is not None:
        try:
            with ZipFile(path, 'w', ZIP_DEFLATED) as zip:
                print(':: converting archive: [%s.files] ...' % root)
                with TarFile.open(fileobj=archive) as tar:
                    for info in tar:
                        if info.isfile() and info.name.endswith('/files'):
                            state = 0
                            lines = [b'']
                            stream = tar.extractfile(info)
                            for line in stream:
                                if line.isspace():
                                    state = 0
                                elif state == 1:
                                    lines.append(line)
                                elif state == 2:
                                    continue
                                elif line.startswith(b'%FILES%'):
                                    state = 1
                                elif line.startswith(b'%BACKUP%'):
                                    state = 2
                            lines = b'/'.join(sorted(lines))
                            if len(lines) > 1:
                                lines = b'\n' + lines
                            zip.writestr(info.name.split('/')[0], lines)
                            stream.close()
                zip.comment = comment
                archive.close()
            return True
        except (EnvironmentError, TarError, UnicodeError) as exception:
            print(':: ERROR: failed to convert archive: '
                  '[%s.files]' % root)
            print('::  ', exception)
            try:
                os.remove(path)
            except OSError:
                pass
    else:
        print(':: ERROR: could not find a valid mirror: '
              '[%s.files]' % root)
    return False

def update_cache(root, comment=''):
    try:
        pool = None
        try:
            config = read_config()
        except IOError as exception:
            print(':: ERROR: could not read pacman config:')
            print('::  ', exception)
        else:
            if not os.path.exists(root):
                print(':: creating cache directory:', root)
                try:
                    os.makedirs(root)
                except OSError as exception:
                    print(':: ERROR: could not create cache directory:')
                    print('::  ', exception)
                    return 1
            args = []
            if not isinstance(comment, bytes):
                comment = comment.encode('utf-8')
            for name in config['Repositories']:
                mirrors = config['Servers'].get(name, [])
                urls = [os.path.join(mirror, '%s.files.tar.gz' % name)
                        for mirror in mirrors]
                path = os.path.join(root, '%s.files.zip' % name)
                args.append((path, name, urls, comment))
            if args:
                def initializer():
                    signal.signal(signal.SIGINT, signal.SIG_IGN)
                pool = Pool(initializer=initializer)
                start = time.time()
                if all(pool.map_async(_process_archive, args).get(1000)):
                    seconds = time.time() - start
                    print(':: update completed in %.1f seconds' % seconds)
                    return 0
            else:
                print(':: ERROR: could not find repositories/mirrors')
    except (KeyboardInterrupt, TimeoutError):
        print(':: update aborted')
        if pool is not None:
            pool.terminate()
            pool.join()
    return 1



# Copyright (C) 2010-2022, kachelaqa <kachelaqa@gmail.com>

import urllib.parse


def make_url(url, query=()):
    parts = urllib.parse.urlparse(url)
    return urllib.parse.urlunparse((
        parts.scheme, parts.netloc,
        urllib.parse.quote(parts.path), urllib.parse.quote(parts.params),
        urllib.parse.urlencode(query, True), ''))
